package junit.logswingui;
/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.<p>
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.<p>
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA<br>
 * http://www.gnu.org/copyleft/lesser.html
 */

import java.awt.BorderLayout;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import junit.logswingui.TestProtocolView;

// Configuration Management Information: 
// -------------------------------------
// $Id: DefaultTestProtocolView.java,v 1.3 2001/11/17 22:42:55 wreissen Exp $
//
// Version History:
// ----------------
// $Log: DefaultTestProtocolView.java,v $
// Revision 1.3  2001/11/17 22:42:55  wreissen
// saving dialog suggests a filename of the form
// JUnit-Protocol-yyyy-MM-dd-nr.txt
//
// Revision 1.2  2001/11/17 11:43:38  wreissen
// protocol saving implemented.
//
// Revision 1.1  2001/11/12 17:54:29  wreissen
// initial version.
//
//
// ***********************************************************************************
/**
 * <p>DefaultTestProtocolView.java</p>
 *
 *
 * Created: Mon Nov 12 14:49:16 2001
 *
 * @author <a href="mailto:wolfgang@openfuture.de">Wolfgang Reissenberger</a>
 * @version $Revision: 1.3 $
 */

public class DefaultTestProtocolView extends JDialog implements TestProtocolView  {
    

    private JTextArea textArea;
    private JFileChooser chooser;
    

    

    /**
     * Creates a new <code>DefaultTestProtocolView</code> instance.
     *
     * @param owner owner frame
     * @param title dialog title
     * @param modal is this window modal?
     */
    public DefaultTestProtocolView(Frame owner, String title, boolean modal) {
	super(owner, title, modal);
	init();
    }



    /**
     * Initialize this panel.
     *
     */
    protected void init() {

	getContentPane().setLayout(new BorderLayout());
	
	JTextArea area = new JTextArea();
	area.setRows(20);
	area.setColumns(60);
	area.setTabSize(0);
	area.setEditable(false);
	setTextArea(area);

	JScrollPane scroller = new JScrollPane(area, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

	addWindowListener(
			  new WindowAdapter() {
				  public void windowClosing(WindowEvent e) {
				      dispose();
				  }
			      }
			  );

	JPanel buttonPanel = new JPanel();
	buttonPanel.setLayout(new BorderLayout())
;
	JButton closeButton= new JButton("Close");
	closeButton.addActionListener(
				 new ActionListener() {
					 public void actionPerformed(ActionEvent e) {
					     hide();
					 }
				     }
				 );
	
	setChooser(new JFileChooser(new File(".")));
	getChooser().setFileSelectionMode(JFileChooser.FILES_ONLY);

	JButton saveButton= new JButton("Save");
	saveButton.addActionListener(
				 new ActionListener() {
					 public void actionPerformed(ActionEvent e) {
					     save();
					 }
				     }
				 );
	
	getContentPane().add(scroller, BorderLayout.NORTH);
	buttonPanel.add(closeButton, BorderLayout.EAST);
	buttonPanel.add(saveButton, BorderLayout.WEST);
	getContentPane().add(buttonPanel, BorderLayout.SOUTH);
	pack();
    }



    /**
     * Save the test protocol.
     *
     */
    public void save() {

	// set a new file name
	File file = getUniqueFilename(getChooser().getCurrentDirectory());
	getChooser().setSelectedFile(file);

	// select the file
	int result = getChooser().showSaveDialog(this);

	if (result == JFileChooser.APPROVE_OPTION) {
	    try {
		file = getChooser().getSelectedFile();

		boolean mayWrite = true;
		if (file.exists()) {
		    int answer =
			JOptionPane.showConfirmDialog(this,
						      "Overwrite existing file?",
						      "Question",
						      JOptionPane.YES_NO_OPTION,
						      JOptionPane.QUESTION_MESSAGE);
		    mayWrite = (answer == JOptionPane.YES_OPTION);
		}

		if (mayWrite) {
		    FileWriter writer = new FileWriter(file);

		    writer.write(getTextArea().getText());
		    writer.close();
		    JOptionPane.showMessageDialog(this,
						  "Protocol saved successfully.",
						  "Success",
						  JOptionPane.INFORMATION_MESSAGE);
		} else {
		    JOptionPane.showMessageDialog(this,
						  "Saving cancelled.",
						  "Cancel",
						  JOptionPane.INFORMATION_MESSAGE);
		}
	    } catch (IOException ex) {
		JOptionPane.showConfirmDialog(this, ex.getMessage(),
					      "Warning",
					      JOptionPane.OK_OPTION,
					      JOptionPane.WARNING_MESSAGE);
	    }
	}
    }



    /**
     * Create a new filename in the given directory. The name is
     * constructed following the scheme
     * <code>JUnit-Protocol-yyyy-MM-dd-nr.txt</code>.
     *
     * @param currentDirectory the current directory
     * @return a file that does not exist in the given directory
     */
    public File getUniqueFilename(File currentDirectory) {
	int nr = 1;

	DateFormat format = new SimpleDateFormat("yyyy-MM-dd");
	String dateString = format.format(new Date());

	File file = new File(currentDirectory,
			     "JUnit-Protocol-" + dateString + "-" +
			     nr + ".txt");

	while(file.exists()) {
	    nr++;
	    file = new File(currentDirectory,
			    "JUnit-Protocol-" + dateString + "-" +
			    nr + ".txt");
	}
	return file;
    }

    // implementation of junit.logswingui.TestProtocolView interface

    /**
     * Appends a message to the text area of the panel.
     *
     * @param message the message to be appended
     */
    public void addMessage(String message) {
	getTextArea().append(message);
    }


    /**
     * Clear the protocol window.
     *
     */
    public void clear() {
	getTextArea().setText("");
    }

    /**
     * Get the value of textArea.
     * @return value of textArea.
     */
    public JTextArea getTextArea() {
	return textArea;
    }
    
    /**
     * Set the value of textArea.
     * @param v  Value to assign to textArea.
     */
    public void setTextArea(JTextArea  v) {
	this.textArea = v;
    }


    /**
     * Get the value of chooser.
     * @return value of chooser.
     */
    public JFileChooser getChooser() {
	return chooser;
    }
    
    /**
     * Set the value of chooser.
     * @param v  Value to assign to chooser.
     */
    public void setChooser(JFileChooser  v) {
	this.chooser = v;
    }
    
} // DefaultTestProtocolView
